"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const splToken = __importStar(require("@solana/spl-token"));
const serum = __importStar(require("@project-serum/serum"));
const fs_1 = __importDefault(require("fs"));
const constants_1 = require("../../src/constants");
//
// Script which creates three mints and two serum3 markets relating them
//
function getVaultOwnerAndNonce(market, programId) {
    const nonce = new anchor_1.BN(0);
    // eslint-disable-next-line no-constant-condition
    while (true) {
        try {
            const vaultOwner = web3_js_1.PublicKey.createProgramAddressSync([market.toBuffer(), nonce.toArrayLike(Buffer, 'le', 8)], programId);
            return [vaultOwner, nonce];
        }
        catch (e) {
            nonce.iaddn(1);
        }
    }
}
async function main() {
    Error.stackTraceLimit = 10000;
    const options = anchor_1.AnchorProvider.defaultOptions();
    options.commitment = 'processed';
    options.preflightCommitment = 'finalized';
    const connection = new web3_js_1.Connection(process.env.CLUSTER_URL, options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.PAYER_KEYPAIR, 'utf-8'))));
    console.log(`Admin ${admin.publicKey.toBase58()}`);
    // Make mints
    const mints = await Promise.all(Array(4)
        .fill(null)
        .map(() => splToken.createMint(connection, admin, admin.publicKey, null, 6)));
    // Mint some tokens to the admin
    for (const mint of mints) {
        const tokenAccount = await splToken.createAssociatedTokenAccountIdempotent(connection, admin, mint, admin.publicKey);
        await splToken.mintTo(connection, admin, mint, tokenAccount, admin, 1e15);
    }
    //const mints = [new PublicKey('5aMD1uEcWnXnptwmyfxmTWHzx3KeMsZ7jmiJAZ3eiAdH'), new PublicKey('FijXcDUkgTiMsghQVpjRDBdUPtkrJfQdfRZkr6zLkdkW'), new PublicKey('3tVDfiFQAAT3rqLNMXUaH2p5X5R4fjz8LYEvFEQ9fDYB')]
    // Make serum markets
    const serumMarkets = [];
    const quoteMint = mints[0];
    for (const baseMint of mints.slice(1, 3)) {
        const feeRateBps = 0.25; // don't think this does anything
        const quoteDustThreshold = 100;
        const baseLotSize = 1000;
        const quoteLotSize = 1000;
        const openbookProgramId = constants_1.OPENBOOK_PROGRAM_ID.devnet;
        const market = web3_js_1.Keypair.generate();
        const requestQueue = web3_js_1.Keypair.generate();
        const eventQueue = web3_js_1.Keypair.generate();
        const bids = web3_js_1.Keypair.generate();
        const asks = web3_js_1.Keypair.generate();
        const baseVault = web3_js_1.Keypair.generate();
        const quoteVault = web3_js_1.Keypair.generate();
        const [vaultOwner, vaultSignerNonce] = getVaultOwnerAndNonce(market.publicKey, openbookProgramId);
        await splToken.createAccount(connection, admin, baseMint, vaultOwner, baseVault);
        await splToken.createAccount(connection, admin, quoteMint, vaultOwner, quoteVault);
        const tx = new web3_js_1.Transaction();
        tx.add(web3_js_1.SystemProgram.createAccount({
            fromPubkey: admin.publicKey,
            newAccountPubkey: market.publicKey,
            lamports: await connection.getMinimumBalanceForRentExemption(serum.Market.getLayout(openbookProgramId).span),
            space: serum.Market.getLayout(openbookProgramId).span,
            programId: openbookProgramId,
        }), web3_js_1.SystemProgram.createAccount({
            fromPubkey: admin.publicKey,
            newAccountPubkey: requestQueue.publicKey,
            lamports: await connection.getMinimumBalanceForRentExemption(5120 + 12),
            space: 5120 + 12,
            programId: openbookProgramId,
        }), web3_js_1.SystemProgram.createAccount({
            fromPubkey: admin.publicKey,
            newAccountPubkey: eventQueue.publicKey,
            lamports: await connection.getMinimumBalanceForRentExemption(262144 + 12),
            space: 262144 + 12,
            programId: openbookProgramId,
        }), web3_js_1.SystemProgram.createAccount({
            fromPubkey: admin.publicKey,
            newAccountPubkey: bids.publicKey,
            lamports: await connection.getMinimumBalanceForRentExemption(65536 + 12),
            space: 65536 + 12,
            programId: openbookProgramId,
        }), web3_js_1.SystemProgram.createAccount({
            fromPubkey: admin.publicKey,
            newAccountPubkey: asks.publicKey,
            lamports: await connection.getMinimumBalanceForRentExemption(65536 + 12),
            space: 65536 + 12,
            programId: openbookProgramId,
        }), serum.DexInstructions.initializeMarket({
            market: market.publicKey,
            requestQueue: requestQueue.publicKey,
            eventQueue: eventQueue.publicKey,
            bids: bids.publicKey,
            asks: asks.publicKey,
            baseVault: baseVault.publicKey,
            quoteVault: quoteVault.publicKey,
            baseMint,
            quoteMint,
            baseLotSize: new anchor_1.BN(baseLotSize),
            quoteLotSize: new anchor_1.BN(quoteLotSize),
            feeRateBps,
            vaultSignerNonce,
            quoteDustThreshold: new anchor_1.BN(quoteDustThreshold),
            programId: openbookProgramId,
            authority: undefined,
        }));
        await (0, web3_js_1.sendAndConfirmTransaction)(connection, tx, [
            admin,
            market,
            requestQueue,
            eventQueue,
            bids,
            asks,
        ]);
        serumMarkets.push(market.publicKey);
    }
    console.log("MINTS='[" + mints.map((pk) => '"' + pk.toBase58() + '"').join(',') + "]'");
    console.log("SERUM_MARKETS='[" +
        serumMarkets.map((pk) => '"' + pk.toBase58() + '"').join(',') +
        "]'");
    process.exit();
}
main();
